// ============================================================================
// Sample by Rashid Thadha Aug 2001
// ============================================================================

#include "stdafx.h"
#include "BandEditCtrl.h"

#include "ToolBand.h"
#include "ToolBandObj.h"

// CBandEditCtrl
CBandEditCtrl::CBandEditCtrl() : m_cRef(0), m_pWebBrowser(NULL)
{
}

CBandEditCtrl::~CBandEditCtrl()
{
}

// Because the host owns the message processing, it will get all keyboard messages
// first.
// 1. We want to use the ENTER key to signal that the user is done in the edit control,
//    so we need to watch for VK_RETURN on WM_KEYUP
// 2. We also want the BACKSPACE, DELETE, END, ARROW keys to go to the edit control and
//	  not to the standard IE interfaces. We translate/dispatch these messages ourselves.
STDMETHODIMP CBandEditCtrl::TranslateAcceleratorIO(LPMSG pMsg)
{
	int nVirtKey = (int)(pMsg->wParam);
	if (WM_KEYUP == pMsg->message && VK_RETURN == nVirtKey)
	{
		CString strNewItem;
		TCHAR cValue[1024];
		GetWindowText(cValue, sizeof(cValue));
		CString strEdit(cValue);

	    _variant_t varURL = _bstr_t(strEdit);
	    _variant_t varEmpty;
	    m_pWebBrowser->Navigate2(&varURL, &varEmpty, &varEmpty, &varEmpty, &varEmpty);

		return S_OK;
	}
	else if ((WM_KEYUP == pMsg->message || WM_KEYDOWN == pMsg->message) &&
				VK_BACK == nVirtKey || (VK_END <= nVirtKey && VK_DOWN >= nVirtKey) ||
				VK_DELETE == nVirtKey)
	{

		TranslateMessage(pMsg);
		DispatchMessage(pMsg);

		return S_OK;
	}
	else
		return S_FALSE;
}

LRESULT CBandEditCtrl::OnSetFocus(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	m_pBand->FocusChange(true);
	
    return 0;
}

LRESULT CBandEditCtrl::OnChar(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	// stop the dinging sound when you press the return key !!!!
    bHandled = (wParam == VK_RETURN);

	return 0;
}
        
STDMETHODIMP CBandEditCtrl::QueryInterface(REFIID riid, VOID** ppv)   
{     
	*ppv=NULL;      
	if (IID_IUnknown==riid || IID_IDropTarget==riid)         
		*ppv=this;      
	if (NULL!=*ppv) 
    {         
		((LPUNKNOWN)*ppv)->AddRef();         
		return NOERROR;         
	}      
	
    return E_NOINTERFACE;
}   

STDMETHODIMP_(ULONG) CBandEditCtrl::AddRef(void)     
{     
	return ++m_cRef;     
}  

STDMETHODIMP_(ULONG) CBandEditCtrl::Release(void)   
{      
	if (0!=--m_cRef)         
		return m_cRef;      	
    delete this;     
	return 0;     
}      


STDMETHODIMP CBandEditCtrl::DragEnter(LPDATAOBJECT pDataObj , DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect)    
{     
	m_pIDataObject=NULL;
	CString sText;
	
    FORMATETC formatDetails;     
	STGMEDIUM stmg;     

	// Let's get the text from the "clipboard"
 	formatDetails.cfFormat = CF_TEXT;
	formatDetails.dwAspect = DVASPECT_CONTENT;
	formatDetails.lindex = -1;
	formatDetails.ptd = NULL;
	formatDetails.tymed = TYMED_HGLOBAL;

	if (SUCCEEDED(pDataObj->GetData(&formatDetails, &stmg)))
    {
	    sText = (LPSTR)GlobalLock(stmg.hGlobal);
	    GlobalUnlock(stmg.hGlobal);

	    if (stmg.hGlobal != NULL) 
		    ReleaseStgMedium(&stmg);             
    }

	*pdwEffect=DROPEFFECT_COPY;

	m_pIDataObject=pDataObj;     
	m_pIDataObject->AddRef();      
	
	return NOERROR;     
}       
        
STDMETHODIMP CBandEditCtrl::DragOver(DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect)   
{     
	if (m_pIDataObject == NULL) 
		*pdwEffect=DROPEFFECT_NONE;         
	else 
		*pdwEffect=DROPEFFECT_COPY;

	return NOERROR;     
}       

STDMETHODIMP CBandEditCtrl::DragLeave(void)     
{     
	m_pIDataObject->Release();
	return NOERROR;     
}      

STDMETHODIMP CBandEditCtrl::Drop(LPDATAOBJECT pDataObj, DWORD grfKeyState, POINTL pt, LPDWORD pdwEffect)     
{     
	*pdwEffect=DROPEFFECT_NONE;      
	if (m_pIDataObject == NULL)         
		return E_FAIL;

	DragLeave();      
	
    FORMATETC formatDetails;     
	STGMEDIUM stmg;     

	// Let's get the text from the "clipboard"
 	formatDetails.cfFormat = CF_TEXT;
	formatDetails.dwAspect = DVASPECT_CONTENT;
	formatDetails.lindex = -1;
	formatDetails.ptd = NULL;
	formatDetails.tymed = TYMED_HGLOBAL;

	if (SUCCEEDED(pDataObj->GetData(&formatDetails, &stmg)))
    {	
	    LPTSTR sClipboard = (LPTSTR)GlobalLock(stmg.hGlobal);
	    SetWindowText(sClipboard);
	    GlobalUnlock(stmg.hGlobal);

	    if (stmg.hGlobal != NULL) 
		    ReleaseStgMedium(&stmg);         
    }
    
	// Navigate
    CString strNewItem;
	TCHAR cValue[1024];
	GetWindowText(cValue, sizeof(cValue));
	CString strEdit(cValue);

	_variant_t varURL = _bstr_t(strEdit);
	_variant_t varEmpty;
	m_pWebBrowser->Navigate2(&varURL, &varEmpty, &varEmpty, &varEmpty, &varEmpty);

	*pdwEffect=DROPEFFECT_COPY;      
	return NOERROR;     
} 

LRESULT CBandEditCtrl::OnKeyUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	if (m_pBand && m_pBand->GetToolBarCtrl())
    {
        int nState = m_pBand->GetToolBarCtrl()->GetState(ID_BUTTON_GREEN);
	    if (nState != -1)
        {
		    char cText[2048];
            GetWindowText(cText, 2048);
            CString sText(cText);
            m_pBand->GetToolBarCtrl()->SetState(ID_BUTTON_GREEN, sText.GetLength() ? nState | TBSTATE_ENABLED : nState & ~TBSTATE_ENABLED);
            m_pBand->GetToolBarCtrl()->SetState(ID_BUTTON_RED, sText.GetLength() ? nState | TBSTATE_ENABLED : nState & ~TBSTATE_ENABLED);
        }
    }

	return 0;
}
